#include "glwidget.h"

#include <QOpenGLBuffer>
#include <QOpenGLShaderProgram>
#include <QPainter>

GLWidget::GLWidget(QWidget *parent) :
    QOpenGLWidget(parent),
    m_program(0),
    m_vertexVBO(0),
    m_colorVBO(0){
}

GLWidget::~GLWidget() {
    delete m_program;
    delete m_vertexVBO;
    delete m_colorVBO;
}

void GLWidget::initializeGL() {
    // Creiamo l'oggetto QOpenGLShaderProgram
    m_program = new QOpenGLShaderProgram(this);

    // Compiliamo lo shader dei vertici
    if (!m_program->addShaderFromSourceFile(QOpenGLShader::Vertex, ":/vertex.glsl"))
        close();

    // Compiliamo il fragment shader
    if (!m_program->addShaderFromSourceFile(QOpenGLShader::Fragment, ":/fragment.glsl"))
        close();

    // Linkiamo gli shaders
    if (!m_program->link())
        close();

    // **** Crea i Vertex Buffer Object ****

    // VBO dei vertici del triangolo
    GLfloat vertexData[] = {
        GLfloat(0.0),  GLfloat(0.707), GLfloat(0.0),  GLfloat(1.0),
        GLfloat(-0.5), GLfloat(-0.50), GLfloat(0.0), GLfloat(1.0),
        GLfloat(0.5),  GLfloat(-0.50), GLfloat(0.0), GLfloat(1.0)
    };
    m_vertexVBO = new QOpenGLBuffer(QOpenGLBuffer::VertexBuffer);
    m_vertexVBO->create();
    m_vertexVBO->setUsagePattern( QOpenGLBuffer::StaticDraw );
    m_vertexVBO->bind();
    m_vertexVBO->allocate( vertexData, 3 * 4 * sizeof(GLfloat));
    m_vertexVBO->release();

    // VBO dei colori
    GLfloat colorData[] = {
        GLfloat(1.0), GLfloat(0.0), GLfloat(0.0), GLfloat(1.0),
        GLfloat(0.0), GLfloat(1.0), GLfloat(0.0), GLfloat(1.0),
        GLfloat(0.0), GLfloat(0.0), GLfloat(1.0), GLfloat(1.0)
    };
    m_colorVBO = new QOpenGLBuffer(QOpenGLBuffer::VertexBuffer);
    m_colorVBO->create();
    m_colorVBO->setUsagePattern( QOpenGLBuffer::StaticDraw );
    m_colorVBO->bind();
    m_colorVBO->allocate( colorData, 3 * 4 * sizeof(GLfloat));
    m_colorVBO->release();

    // Localizzaziamo gli attributi negli shader
    m_posAttr = m_program->attributeLocation("posAttr");
    m_colAttr = m_program->attributeLocation("colAttr");
    m_matrixUniform = m_program->uniformLocation("matrix");
}

void GLWidget::resizeGL(int w, int h) {
    glViewport(0, 0, w, h);
}

void GLWidget::paintGL() {
    // Usiamo QPainter perché vogliamo usarlo per integrare il disegno opengl
    QPainter painter;

    painter.begin(this);

    painter.beginNativePainting();

    // **** Disegni opengl ****

    // Rendiamo corrente il contesto - in questo caso non è
    // necessario, ma è una buona abitudine
    makeCurrent();

    // Puliamo la la vista
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

    // Linkiamo gli shader
    m_program->bind();

    // Carichiamo la matrice modello/vista
    // impostiamo la prospettiva
    QMatrix4x4 matrix;
    matrix.perspective(60, 4.0/3.0, 0.1, 100.0);
    matrix.translate(0, 0, -2);
    m_program->setUniformValue(m_matrixUniform, matrix);

    // Attiviamo il VBO dei vertici
    m_vertexVBO->bind();
    m_program->enableAttributeArray( m_posAttr );
    m_program->setAttributeBuffer( m_posAttr, GL_FLOAT, 0, 4);
    m_vertexVBO->release();

    // Attiviamo il VBO dei colori
    m_colorVBO->bind();
    m_program->enableAttributeArray( m_colAttr );
    m_program->setAttributeBuffer( m_colAttr, GL_FLOAT, 0, 4);
    m_colorVBO->release();

    // Disegniamo il triangolo
    glDrawArrays(GL_TRIANGLES, 0, 3);

    // Rilasciamo gli shader
    m_program->release();

    painter.endNativePainting();

    painter.end();
}
